@@ -644,7 +644,7 @@ surface_unmap_image (PycairoSurface *self, PyObject *args) {
   /* Replace the mapped image surface with a fake one and finish it so
    * that any operation on it fails.
    */
-  fake_surface = cairo_image_surface_create (CAIRO_FORMAT_INVALID, 0, 0);
+  fake_surface = cairo_image_surface_create (CAIRO_FORMAT_ARGB32, 0, 0);
   cairo_surface_finish (fake_surface);
   pymapped->surface = fake_surface;
   /* We no longer need the base surface */
@@ -655,6 +655,20 @@ surface_unmap_image (PycairoSurface *self, PyObject *args) {
 
 #endif /* CAIRO_HAS_IMAGE_SURFACE */
 
+static PyObject *
+surface_ctx_enter (PyObject *obj) {
+  Py_INCREF (obj);
+  return obj;
+}
+
+static PyObject *
+surface_ctx_exit (PycairoSurface *obj, PyObject *args) {
+  Py_BEGIN_ALLOW_THREADS;
+  cairo_surface_finish (obj->surface);
+  Py_END_ALLOW_THREADS;
+  Py_RETURN_NONE;
+}
+
 static PyMethodDef surface_methods[] = {
   /* methods never exposed in a language binding:
    * cairo_surface_destroy()
@@ -663,6 +677,8 @@ static PyMethodDef surface_methods[] = {
    * cairo_surface_reference()
    * cairo_surface_set_user_data()
    */
+  {"__enter__",      (PyCFunction)surface_ctx_enter,          METH_NOARGS},
+  {"__exit__",       (PyCFunction)surface_ctx_exit,           METH_VARARGS},
   {"copy_page",      (PyCFunction)surface_copy_page,          METH_NOARGS},
   {"create_similar", (PyCFunction)surface_create_similar,     METH_VARARGS},
   {"create_similar_image", (PyCFunction)surface_create_similar_image,
@@ -1124,8 +1140,13 @@ mapped_image_surface_dealloc (PycairoImageSurface *self) {
   if (cairo_surface_get_user_data (
       self->surface, &surface_is_mapped_image) != NULL) {
     cairo_surface_unmap_image (pybasesurface->surface, self->surface);
+    self->surface = NULL;
+  } else {
+    cairo_surface_destroy(self->surface);
+    self->surface = NULL;
   }
 
+  Py_CLEAR (self->base);
   Py_TYPE (self)->tp_free (self);
 }
 
@@ -1137,7 +1158,21 @@ mapped_image_surface_finish (PycairoSurface *self) {
   return NULL;
 }
 
+static PyObject *
+mapped_image_surface_ctx_exit (PycairoImageSurface *self, PyObject *args) {
+  PycairoSurface *pybasesurface = (PycairoSurface *)(self->base);
+  PyObject *subargs, *result;
+
+  subargs = Py_BuildValue("(O)", self);
+  if (subargs == NULL)
+    return NULL;
+  result = surface_unmap_image (pybasesurface, subargs);
+  Py_DECREF (subargs);
+  return result;
+}
+
 static PyMethodDef mapped_image_surface_methods[] = {
+  {"__exit__",     (PyCFunction)mapped_image_surface_ctx_exit,   METH_VARARGS},
   {"finish",       (PyCFunction)mapped_image_surface_finish,     METH_NOARGS},
   {NULL, NULL, 0, NULL},
 };
