@@ -13,6 +13,14 @@ import cairo
 import pytest
 
 
+def test_context_manager():
+    with cairo.ImageSurface(cairo.FORMAT_ARGB32, 10, 10) as surface:
+        surface.show_page()
+    with pytest.raises(cairo.Error) as excinfo:
+        surface.show_page()
+    assert excinfo.value.status == cairo.Status.SURFACE_FINISHED
+
+
 def test_surface_cmp_hash():
     main = cairo.ImageSurface(cairo.FORMAT_ARGB32, 10, 10)
     ctx = cairo.Context(main)
@@ -66,6 +74,23 @@ def test_surface_map_to_image():
     del gced
 
 
+def test_surface_map_to_image_context_manager():
+    main = cairo.ImageSurface(cairo.FORMAT_ARGB32, 10, 10)
+    with main.map_to_image(None) as image:
+        pass
+
+    with pytest.raises(RuntimeError):
+        main.unmap_image(image)
+
+    with pytest.raises(cairo.Error) as excinfo:
+        image.show_page()
+    assert excinfo.value.status == cairo.Status.SURFACE_FINISHED
+
+    with pytest.raises(RuntimeError):
+        with image:
+            pass
+
+
 def test_surface_map_to_image_data():
     main = cairo.ImageSurface(cairo.Format.RGB24, 2, 1)
 
@@ -389,6 +414,7 @@ def test_supports_mime_type():
         surface.supports_mime_type(object())
 
 
+# https://bitbucket.org/pypy/pypy/issues/2751
 @pytest.mark.skipif(platform.python_implementation() == "PyPy", reason="PyPy")
 def test_image_surface_create_for_data_array():
     width, height = 255, 255
@@ -402,7 +428,7 @@ def test_image_surface_create_for_data_array():
             b = int(x * alpha / 255.0)
             g = int(y * alpha / 255.0)
             # cairo.FORMAT_ARGB32 uses pre-multiplied alpha
-            data[offset:offset + 3] = array.array(
+            data[offset:offset + 4] = array.array(
                 "B", struct.pack("=I", b | g << 8 | alpha << 24))
 
     surface = cairo.ImageSurface.create_for_data(
